@@ -4,7 +4,7 @@ use queues::*;
 use rand::prelude::*;
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
 
-use crate::error::CNErr;
+use crate::error::CnErr;
 use crate::FlagContent;
 use crate::Network;
 
@@ -15,7 +15,7 @@ use crate::Network;
 /// the new root, and the process is repeated until there are no more unexplored nodes.
 /// # Examples
 /// ```
-/// use cnetworks::*;
+/// # use cnetworks::*;
 /// let net = Network::new(100, Model::ER { p: 0.05, whole: false }, Weight::None );
 /// let clusters = clusters(&net);
 /// println!("Largest cluster: {:?}", clusters.first().unwrap());
@@ -58,14 +58,14 @@ pub fn clusters(net: &Network) -> Vec<HashSet<usize>> {
 ///
 /// # Examples
 /// ```
-/// use cnetworks::*;
+/// # use cnetworks::*;
 /// let net = Network::new(100, Model::ER { p: 0.05, whole: false }, Weight::None);
 /// match bfs(&net, 1, 3).unwrap() {
 ///     Some(d) => println!("Edges between 1 and 3: {}", d),
 ///     None => println!("No path from 1 to 3 found!"),
 /// }
 /// ```
-pub fn bfs(net: &Network, root: usize, target: usize) -> Result<Option<usize>, CNErr> {
+pub fn bfs(net: &Network, root: usize, target: usize) -> Result<Option<usize>, CnErr> {
     bfs_callback(net, root, target, |_, _, _| ())
 }
 
@@ -82,24 +82,26 @@ pub fn bfs(net: &Network, root: usize, target: usize) -> Result<Option<usize>, C
 ///
 /// # Examples
 /// ```
-/// use cnetworks::*;
+/// # use cnetworks::*;
 /// let net = Network::new(100, Model::ER { p: 0.05, whole: false }, Weight::None);
+/// let mut visited = Vec::new();
 /// match bfs_callback(
 ///     &net,
 ///     1,
 ///     3,
-///     |current: usize, parent: Option<usize>, children: &Vec<&usize>| { /* Do something... */ },
-/// ).unwrap() {
-///     Some(n) => println!("Found 3 after discovering {:?}", n),
-///     None => println!("No path to 3 found!"),
+///     |current: usize, parent: Option<usize>, children: &Vec<&usize>| visited.push(current)
+///     ).unwrap() {
+///     Some(d) => println!("Edges between 1 and 3: {}", d),
+///     None => println!("No path from 1 to 3 found!"),
 /// }
+/// println!("Visited {:?} in the process.", visited);
 /// ```
 pub fn bfs_callback<F: FnMut(usize, Option<usize>, &Vec<&usize>)>(
     net: &Network,
     root: usize,
     target: usize,
     mut step_callback: F,
-) -> Result<Option<usize>, CNErr> {
+) -> Result<Option<usize>, CnErr> {
     // Check if the target and root are valid nodes
     net.get(target)?;
     net.get(root)?;
@@ -151,13 +153,13 @@ pub fn bfs_callback<F: FnMut(usize, Option<usize>, &Vec<&usize>)>(
 ///
 /// # Examples
 /// ```
-/// use cnetworks::*;
+/// # use cnetworks::*;
 /// let net = Network::new(100, Model::ER { p: 0.05, whole: false }, Weight::None);
 /// let (exp, unexp) = explore(&net, 1).unwrap();
 /// println!("Available from 1: {:?}", exp);
 /// println!("Unavailable from 1: {:?}", unexp);
 /// ```
-pub fn explore(net: &Network, root: usize) -> Result<(HashSet<usize>, HashSet<usize>), CNErr> {
+pub fn explore(net: &Network, root: usize) -> Result<(HashSet<usize>, HashSet<usize>), CnErr> {
     net.get(root)?;
     let mut discovered: HashSet<usize> = HashSet::default();
     let mut to_visit: Queue<usize> = Queue::new();
