@@ -16,6 +16,7 @@ extern crate rand;
 extern crate rand_xoshiro;
 extern crate serde;
 extern crate rustc_hash;
+extern crate rand_seeder;
 
 mod algorithms;
 mod error;
@@ -24,7 +25,7 @@ mod node;
 mod flag;
 
 pub use algorithms::*;
-pub use error::CNErr;
+pub use error::CnErr;
 pub use network::*;
 pub use node::*;
 pub use flag::*;
