@@ -1,13 +1,14 @@
 //! This module contains methods for `Network` related to creating and removing links as well as
 //! disconnecting entire nodes from the network.
 
-use crate::algorithms::bfs;
-use crate::error::CNErr;
 use super::models::Model;
 use super::Network;
+use crate::algorithms::bfs;
+use crate::error::CnErr;
+use crate::Node;
 use rand::prelude::*;
 use rustc_hash::FxHashMap as HashMap;
-use serde::{Serialize, Deserialize};
+use serde::{Deserialize, Serialize};
 
 /// `Weight` determines the weight of each link in the network (ie. the probability of the
 /// infection spreading through it in each time step).
@@ -29,23 +30,24 @@ impl Default for Weight {
 }
 
 impl Network {
-    /// Get the weight of the link between nodes i and j.
+    /// Get the weight of the link between nodes `i` and `j`.
     ///
-    /// Returns `None` if such link does not exist and `Err` if i and j are the same or do not
+    /// Returns `None` if such link does not exist and `Err` if `i` and `j` are the same or do not
     /// exist.
-    pub fn get_link(&self, i: usize, j: usize) -> Result<Option<f64>, CNErr> {
+    pub fn get_link(&self, i: usize, j: usize) -> Result<Option<f64>, CnErr> {
         let node_i = self.get(i)?;
         self.get(j)?;
         if i == j {
-            return Err(CNErr::LinkToSelf(i));
+            return Err(CnErr::LinkToSelf(i));
         }
         Ok(node_i.links().get(&j).cloned())
     }
 
-    /// Establish a link between nodes i and j with current network's weight.  Returns the weight
-    /// if the requested link already exist, `None` if it does not. Returns `Err` if i
-    /// and j are the same or do not exist.
-    pub fn link(&mut self, i: usize, j: usize) -> Result<Option<f64>, CNErr> {
+    /// Establish a link between nodes `i` and `j` using current network's weight.
+    ///
+    /// Returns the weight if the requested link already exist, `None` if it does not. Returns
+    /// `Err` if `i` and `j` are the same or do not exist.
+    pub fn link(&mut self, i: usize, j: usize) -> Result<Option<f64>, CnErr> {
         let w = match self.weight {
             Weight::Constant { c } => c,
             Weight::Uniform => self.rng().unwrap().gen(),
@@ -54,19 +56,21 @@ impl Network {
         self.link_exact(i, j, w)
     }
 
-    /// Establish a link between nodes i and j with specified `weight`. Updates the link weight and
-    /// returns the old value if it already exist, `None` otherwise. Returns `Err` if i
-    /// and j are the same or do not exist, or if the weight is not in the range (0, 1].
-    pub fn link_exact(&mut self, i: usize, j: usize, w: f64) -> Result<Option<f64>, CNErr> {
+    /// Establish a link between nodes i and j with specified `weight`.
+    ///
+    /// Updates the link weight and returns the old value if it already exist, `None` otherwise.
+    /// Returns `Err` if `i` and `j` are the same or do not exist, or if the `weight` is not in the range
+    /// (0, 1].
+    pub fn link_exact(&mut self, i: usize, j: usize, weight: f64) -> Result<Option<f64>, CnErr> {
         let ret = self.get_link(i, j)?;
-        if w <= 0. || w > 1. {
-            return Err(CNErr::BadWeight(w));
+        if weight <= 0. || weight > 1. {
+            return Err(CnErr::BadWeight(weight));
         }
         if i == j {
-            return Err(CNErr::LinkToSelf(i));
+            return Err(CnErr::LinkToSelf(i));
         }
-        self.get_mut(i).unwrap().link_to(j, w);
-        self.get_mut(j).unwrap().link_to(i, w);
+        self.get_mut(i).unwrap().link_to(j, weight);
+        self.get_mut(j).unwrap().link_to(i, weight);
         Ok(ret)
     }
 
@@ -74,8 +78,8 @@ impl Network {
     /// the network is preserved.
     ///
     /// Returns the weight of the removed connection or `None` if the connection does not exist and
-    /// `Err` if i and j are the same or do not exist.
-    pub fn unlink(&mut self, i: usize, j: usize) -> Result<Option<f64>, CNErr> {
+    /// `Err` if `i` and `j` are the same or do not exist.
+    pub fn unlink(&mut self, i: usize, j: usize) -> Result<Option<f64>, CnErr> {
         match self.get_link(i, j)? {
             Some(weight) => {
                 self.get_mut(i).unwrap().unlink_from(j);
@@ -91,7 +95,7 @@ impl Network {
     ///
     /// Returns the weight of removed link or `None` if the link does not exist or cannot be safely
     /// removed. Returns `Err` if i and j are the same or do not exist.
-    pub fn unlink_safe(&mut self, i: usize, j: usize) -> Result<Option<f64>, CNErr> {
+    pub fn unlink_safe(&mut self, i: usize, j: usize) -> Result<Option<f64>, CnErr> {
         let w = match self.unlink(i, j)? {
             Some(weight) => weight,
             None => return Ok(None),
@@ -109,9 +113,9 @@ impl Network {
     /// Unsafely disconnect a node from the network by removing all of its links. Does not
     /// check if the network integrity is perserved.
     ///
-    /// Returns the removed links as a `HashMap` of (target, weight) pairs. Returns `Err` if i
-    /// and j are the same or do not exist.
-    pub fn disconnect(&mut self, node: usize) -> Result<HashMap<usize, f64>, CNErr> {
+    /// Returns the removed links as a `HashMap` of (target, weight) pairsm `Err` if `i`
+    /// and `j` are the same or do not exist.
+    pub fn disconnect(&mut self, node: usize) -> Result<HashMap<usize, f64>, CnErr> {
         let node = self.get_mut(node)?;
         let old_links = node.links().clone();
         node.disconnect();
@@ -127,8 +131,10 @@ impl Network {
     /// the network is perserved.
     ///
     /// Returns removed links as `HashMap` of (target, weight) pairs or `None` if `node` cannot be
-    /// safely disconnected. Returns `Err` if `node` does not exist.
-    pub fn disconnect_safe(&mut self, node: usize) -> Result<Option<HashMap<usize, f64>>, CNErr> {
+    /// safely disconnected.
+    ///
+    /// Returns `Err` if `node` does not exist.
+    pub fn disconnect_safe(&mut self, node: usize) -> Result<Option<HashMap<usize, f64>>, CnErr> {
         // Blindly disconnect the requested node
         let links = self.disconnect(node)?;
         // Get a vector of used-to-be-connected nodes
@@ -161,4 +167,55 @@ impl Network {
         }
         self.model = Model::None;
     }
+
+    /// Attach new nodes with specified indexes. Note that the nodes must be connected to the rest
+    /// of the network manually, as no links will be automatically established.
+    ///
+    /// Returns `Err` if there already exists a node with any of the specified indexes. No nodes
+    /// will be attached in that case.
+    pub fn attach(&mut self, indexes: &[usize]) -> Result<(), CnErr> {
+        // Check for existance first
+        for &i in indexes {
+            if self.get(i).is_ok() {
+                return Err(CnErr::NodeExists(i));
+            }
+        }
+        // Only attach if all idexes in slice are unoccupied.
+        for &i in indexes {
+            self.nodes.push(Node::new(i));
+            self.size += 1;
+        }
+        self.nodes.sort_unstable_by_key(|node| *node.index());
+        let mut indexing_ok = self.indexing_ok;
+        if indexing_ok {
+            // Reverse the iter since beggining nodes were likely added during network creation
+            for (true_index, node) in self.nodes().enumerate().rev() {
+                if node.index() != &true_index {
+                    indexing_ok = false;
+                    break;
+                }
+            }
+        }
+        self.indexing_ok = indexing_ok;
+        Ok(())
+    }
+
+    /// Completely remove the node with specified `index` from the network, disconnecting it first.
+    ///
+    /// Returns `Err` if node with specified index does not exist.
+    pub fn detach(&mut self, index: usize) -> Result<(), CnErr> {
+        self.disconnect(index)?;
+        if self.indexing_ok {
+            self.nodes.remove(index);
+        } else {
+            let true_index = self
+                .nodes
+                .binary_search_by_key(&index, |node| *node.index())
+                .unwrap(); // We can unwrap since the disconnect already checks for existance
+            self.nodes.remove(true_index);
+        }
+        self.size -= 1;
+        self.indexing_ok = false;
+        Ok(())
+    }
 }
