@@ -1,9 +1,10 @@
 //! This module contains the main `Network` graph-like data structure.
-mod link;
-mod models;
 mod new;
 mod props;
 
+mod link;
+mod models;
+
 pub use link::*;
 pub use models::Model;
 pub use new::*;
@@ -27,9 +28,11 @@ pub struct Network {
     weight: Weight,
     /// The network's randomness source
     rng: RefCell<Xoshiro256Plus>,
+    /// The [`Network::rng`] seed
+    seed: String,
     /// The network size (total number of nodes in the network)
     size: usize,
-    /// The network indexing integrity, ie. whether nodes[i].index() == i.
+    /// The network indexing integrity, ie. whether nodes\[i\].index() == i.
     indexing_ok: bool,
 }
 
