@@ -4,7 +4,7 @@ use super::link::Weight;
 use super::Model;
 use super::Network;
 
-use crate::CNErr;
+use crate::CnErr;
 use crate::Node;
 
 use core::cell::{BorrowMutError, RefMut};
@@ -13,8 +13,8 @@ use rustc_hash::FxHashMap as HashMap;
 
 impl Network {
     /// Return the network size, ie. the total number of nodes.
-    pub fn size(&self) -> usize {
-        self.size
+    pub fn size(&self) -> &usize {
+        &self.size
     }
 
     /// Return the model of the network
@@ -27,35 +27,64 @@ impl Network {
         &self.weight
     }
 
-    /// Try to borrow the network-local rng thread from it's `RefCell`. This is effectively to
-    /// calling a `RefCell::try_borrow_mut()`.
+    /// Return the network-local rng thread seed.
+    ///
+    /// This is a random alphanumeric string of length 16, unless explicitly specified in
+    /// [`Network::with_seed()`].
+    pub fn seed(&self) -> &str {
+        &self.seed
+    }
+
+    /// Try to borrow the network-local rng thread from it's `RefCell`. This is effectively
+    /// calling `RefCell::try_borrow_mut()`.
     pub fn rng(&self) -> Result<RefMut<'_, impl RngCore>, BorrowMutError> {
         self.rng.try_borrow_mut()
     }
 
-    /// Return iterator containing the node indexes. Use `nodes()` to iterate over the nodes
-    /// themselves and `enumerated()` to use both indexes and values at once.
-    pub fn indexes(&self) -> impl Iterator<Item = usize> + '_ {
-        self.nodes.iter().map(|node| *node.index())
-    }
-
-    /// Return iterator containing the network nodes.
-    pub fn nodes(&self) -> impl Iterator<Item = &Node> {
-        self.nodes.iter()
-    }
-
     /// Return iterator containing the (index, node) pairs.
-    pub fn enumerated(&self) -> impl Iterator<Item = (usize, &Node)> + '_ {
+    ///
+    /// Note that this is **not** strictly equivalent to `Network::nodes().enumerate()`, since a
+    /// node's index **does not**  always equal it's position in the iterator, even though nodes
+    /// are always ordered by their index.
+    ///
+    /// For example it is possible to have a network with indexes (0, 2, 3) or (100, 101,
+    /// 150), but not (10, 0, 1).
+    pub fn iter(
+        &self,
+    ) -> impl DoubleEndedIterator<Item = (usize, &Node)> + ExactSizeIterator<Item = (usize, &Node)> + '_
+    {
         self.nodes.iter().map(|node| (*node.index(), node))
     }
 
-    /// Returns immutable reference to a given node if it exists, `Err` if it does not.
-    pub fn get(&self, index: usize) -> Result<&Node, CNErr> {
+
+    /// Return iterator containing the node indexes.
+    ///
+    /// Use [`Network::nodes()`] to iterate over the nodes themselves and [`Network::iter()`] to use both indexes
+    /// and values at once.
+    ///
+    /// Keep in mind that nodes are **always** ordered by their index, but it's possible to have a
+    /// network of nodes with non-sequential indexes, such as (0,2,3) or (100, 101, 150), but not
+    /// (10, 0 1).
+    pub fn indexes(
+        &self,
+    ) -> impl DoubleEndedIterator<Item = usize> + ExactSizeIterator<Item = usize> + '_ {
+        self.nodes.iter().map(|node| *node.index())
+    }
+
+    /// Return iterator containing the network nodes, ordered by their index.
+    pub fn nodes(
+        &self,
+    ) -> impl DoubleEndedIterator<Item = &Node> + ExactSizeIterator<Item = &Node> {
+        self.nodes.iter()
+    }
+
+    /// Return immutable reference to a given node if it exists, `Err` if it does not.
+    pub fn get(&self, index: usize) -> Result<&Node, CnErr> {
         if self.indexing_ok {
             // The node can be found at the specified index
             match self.nodes.get(index) {
                 Some(node) => Ok(node),
-                None => Err(CNErr::NoSuchNode(index)),
+                None => Err(CnErr::NoSuchNode(index)),
             }
         } else {
             // Binary search is required
@@ -64,18 +93,18 @@ impl Network {
                 .binary_search_by_key(&index, |node| *node.index())
             {
                 Ok(true_idx) => Ok(&self.nodes[true_idx]),
-                Err(..) => Err(CNErr::NoSuchNode(index)),
+                Err(..) => Err(CnErr::NoSuchNode(index)),
             }
         }
     }
 
-    /// Returns mutable reference to a given node if it exists, `Err` if it does not.
-    pub(crate) fn get_mut(&mut self, index: usize) -> Result<&mut Node, CNErr> {
+    /// Return mutable reference to a given node if it exists, `Err` if it does not.
+    pub(crate) fn get_mut(&mut self, index: usize) -> Result<&mut Node, CnErr> {
         if self.indexing_ok {
             // The node can be found at the specified index
             match self.nodes.get_mut(index) {
                 Some(node) => Ok(node),
-                None => Err(CNErr::NoSuchNode(index)),
+                None => Err(CnErr::NoSuchNode(index)),
             }
         } else {
             // Binary search is required
@@ -84,7 +113,7 @@ impl Network {
                 .binary_search_by_key(&index, |node| *node.index())
             {
                 Ok(true_idx) => Ok(&mut self.nodes[true_idx]),
-                Err(..) => Err(CNErr::NoSuchNode(index)),
+                Err(..) => Err(CnErr::NoSuchNode(index)),
             }
         }
     }
