@@ -3,8 +3,8 @@ use cnetworks::*;
 use rustc_hash::FxHashMap as HashMap;
 
 #[test]
-fn basic_link() {
-    let mut net = Network::new(10, Model::None, Weight::Constant { c: 1.0 });
+fn test_link() {
+    let mut net = Network::new(10, Model::None, Weight::default());
 
     for i in net.indexes() {
         for j in net.indexes() {
@@ -23,28 +23,13 @@ fn basic_link() {
     assert!((net.get_link(2, 3).unwrap().unwrap() - 0.25).abs() < f64::EPSILON);
     assert!((net.get_link(3, 2).unwrap().unwrap() - 0.25).abs() < f64::EPSILON);
 
-    assert_eq!(net.link(1, 100).unwrap_err(), CNErr::NoSuchNode(100));
-    assert_eq!(net.get_link(1, 100).unwrap_err(), CNErr::NoSuchNode(100));
+    assert_eq!(net.link(1, 100).unwrap_err(), CnErr::NoSuchNode(100));
+    assert_eq!(net.get_link(1, 100).unwrap_err(), CnErr::NoSuchNode(100));
 }
 
 #[test]
-fn test_stats() {
-    let mut net = Network::new(3, Model::None, Weight::Constant { c: 1.0 });
-    net.link(0, 1).unwrap();
-    net.link(0, 2).unwrap();
-    assert_eq!(net[0].deg(), &2);
-    assert_eq!(net[1].deg(), &1);
-    assert_eq!(net[2].deg(), &1);
-    assert_eq!(net.total_deg(), 4);
-    assert_eq!(net.edges(), 2);
-
-    let map = vec![(1, 2), (2, 1)].into_iter().collect();
-    assert_eq!(net.deg_distr(), map);
-}
-
-#[test]
-fn basic_unlink() {
-    let mut net = Network::new(3, Model::None, Weight::Constant { c: 1.0 });
+fn test_unlink() {
+    let mut net = Network::new(3, Model::None, Weight::default());
 
     net.link(0, 1).unwrap();
     assert_eq!(net.unlink_safe(0, 1).unwrap(), None);
@@ -52,8 +37,8 @@ fn basic_unlink() {
     assert_eq!(net.get_link(0, 1).unwrap(), None);
     assert_eq!(net.get_link(1, 0).unwrap(), None);
 
-    assert_eq!(net.unlink(0, 100).unwrap_err(), CNErr::NoSuchNode(100));
-    assert_eq!(net.unlink_safe(0, 100).unwrap_err(), CNErr::NoSuchNode(100));
+    assert_eq!(net.unlink(0, 100).unwrap_err(), CnErr::NoSuchNode(100));
+    assert_eq!(net.unlink_safe(0, 100).unwrap_err(), CnErr::NoSuchNode(100));
 
     net.link(0, 1).unwrap();
     net.link(1, 2).unwrap();
@@ -64,8 +49,8 @@ fn basic_unlink() {
 }
 
 #[test]
-fn basic_disconnect() {
-    let mut net = Network::new(3, Model::None, Weight::Constant { c: 1.0 });
+fn test_disconnect() {
+    let mut net = Network::new(3, Model::None, Weight::default());
     let idxes: Vec<_> = net.indexes().collect();
     for &i in &idxes {
         for &j in &idxes {
@@ -78,10 +63,34 @@ fn basic_disconnect() {
         assert!(vec![1, 2].contains(&target));
         assert!((weight - 1.0).abs() < f64::EPSILON);
     }
-    println!("{:?}", net);
+
     assert_eq!(net.get_link(0, 1).unwrap(), None);
     assert_eq!(net.disconnect(0).unwrap(), HashMap::default());
     assert!((net.disconnect(1).unwrap()[&2] - 1.0).abs() < f64::EPSILON);
     assert_eq!(net.disconnect(2).unwrap(), HashMap::default());
-    assert_eq!(net.disconnect(100).unwrap_err(), CNErr::NoSuchNode(100));
+    assert_eq!(net.disconnect(100).unwrap_err(), CnErr::NoSuchNode(100));
+}
+
+#[test]
+fn test_attach_detach() {
+    let mut net = Network::new(0, Model::None, Weight::default());
+    assert_eq!(net.size(), &0);
+
+    net.attach(&[0, 1, 2]).unwrap();
+    assert_eq!(net.size(), &3);
+    assert_eq!(net.size_connected(), 0);
+    assert_eq!(net.indexes().collect::<Vec<_>>(), vec![0,1,2]);
+
+    assert_eq!(net.attach(&[0, 3, 4]).unwrap_err(), CnErr::NodeExists(0));
+    for i in 0..3 {
+        assert!(net.get(i).is_ok());
+    }
+
+    net.detach(2).unwrap();
+    assert_eq!(net.size(), &2);
+    assert_eq!(net.indexes().collect::<Vec<_>>(), vec![0,1]);
+    for i in 0..2 {
+        assert!(net.get(i).is_ok());
+    }
+    assert!(net.get(2).is_err());
 }
